import MyServer, { IData, MyServerEventEnum } from './core';
import Player from './biz/Player';
import PlayerManager from './biz/PlayerManager';
import RoomManager from './biz/RoomManager';
import Connection from './core/Connection';
import { copyCommon, getTime } from './utils';

export enum ApiMsgEnum {
  ApiPlayerList = 'ApiPlayerList',
  ApiPlayerJoin = 'ApiPlayerJoin',
  ApiRoomList = 'ApiRoomList',
  ApiRoomCreate = 'ApiRoomCreate',
  ApiRoomJoin = 'ApiRoomJoin',
  ApiRoomLeave = 'ApiRoomLeave',
  MsgPlayerList = 'MsgPlayerList',
  MsgRoomList = 'MsgRoomList',
  MsgRoom = 'MsgRoom',
}

copyCommon()

const server = new MyServer({ port: 8888 })

// event
server.on(MyServerEventEnum.Connect, (connection: Connection) => {
  console.log(`${getTime()}来人|人数|${server.connections.size}`)
})

server.on(MyServerEventEnum.DisConnect, (connection: Connection) => {
  console.log(`${getTime()}走人|人数|${server.connections.size}`)
  if (connection.playerId) {
    PlayerManager.Instance.removePlayer(connection.playerId)
  }
})

// api
server.setApi(ApiMsgEnum.ApiPlayerList, (connection: Connection, data: IData) => {
  return { list: PlayerManager.Instance.getPlayersView() }
})

server.setApi(ApiMsgEnum.ApiPlayerJoin, (connection: Connection, data: IData) => {
  const nickname = data.nickname
  const player = PlayerManager.Instance.createPlayer({ connection, nickname })
  PlayerManager.Instance.syncPlayers()
  return PlayerManager.Instance.getPlayerView(player)
})

server.setApi(ApiMsgEnum.ApiRoomList, (connection: Connection, data: IData) => {
  return { list: RoomManager.Instance.getRoomsView() }
})

server.setApi(ApiMsgEnum.ApiRoomCreate, (connection: Connection, data: IData) => {
  if (connection.playerId) {
    const room = RoomManager.Instance.joinRoom(RoomManager.Instance.createRoom().id, connection.playerId)
    if (room) {
      RoomManager.Instance.syncRooms()
      PlayerManager.Instance.syncPlayers()
      return RoomManager.Instance.getRoomView(room)
    } else {
      throw new Error("ApiRoomCreate room不存在")
    }
  } else {
    throw new Error("ApiRoomCreate 玩家未登录")
  }
})

server.setApi(ApiMsgEnum.ApiRoomJoin, (connection: Connection, data: IData) => {
  if (connection.playerId) {
    const room = RoomManager.Instance.joinRoom(data.rid, connection.playerId)
    if (room) {
      RoomManager.Instance.syncRooms()
      PlayerManager.Instance.syncPlayers()
      RoomManager.Instance.syncRoom(room.id)
      return RoomManager.Instance.getRoomView(room)
    } else {
      throw new Error("ApiRoomJoin room不存在")
    }
  } else {
    throw new Error("ApiRoomJoin 玩家未登录")
  }
})

server.setApi(ApiMsgEnum.ApiRoomLeave, (connection: Connection, data: IData) => {
  if (connection.playerId) {
    const player = PlayerManager.Instance.getPlayerById(connection.playerId)
    if (player) {
      const rid = player.rid
      if (rid) {
        RoomManager.Instance.leaveRoom(rid, player.id)
        PlayerManager.Instance.syncPlayers()
        RoomManager.Instance.syncRooms()
        RoomManager.Instance.syncRoom(rid)
      } else {
        throw new Error("ApiRoomLeave 玩家不在房间")
      }
    } else {
      throw new Error("ApiRoomLeave 玩家不存在")
    }
  } else {
    throw new Error("ApiRoomLeave 玩家未登录")
  }
})

// start!!
server.start().then(() => {
  console.log("服务启动！")
}).catch((e) => {
  console.log("服务异常", e)
})
